/**************************************************************

NEC IR decoder programma - Bart Huyskens - SJI Schoten - 23/04/2013

Sensor: TSOP1236 - aangesloten aan PIN RB0 (Externe interrupt)
deze sensor inverteert wel het resultaat... rekening mee houden

Timer 1 wordt gebruikt om de tijd tussen twee flanken te meten

- Xtal: 19660800
- Prescaler: 8 - incrrement periode: 1.627usec

situaties:
0 - TMR1 overflow (interrupt) - NO KEY PRESSED +/-== 65536
1 - 13.5 msec tussen 2 neg flanken - START (8294 x 1.627usec) 	7500<time<9000
2 - 11.25 msec tussen 2 neg flanken - REPEAT (6912 x 1.627usec)	5000<time<7499
3 - 2.25 msec tussen 2 neg flanken - "1" (1382 x 1.627usec)		1050<time<1700
4 - 1.125 msec tussen 2 neg flanken - "0" (691 x 1.627usec)		300<time<1000	

***************************************************************/
#include <htc.h> 
#include "HITECH_LCD.h"		// include LCD library
#define _XTAL_FREQ  19660800  

__CONFIG(0x2FF2); // external HS Xtal 19660800
__CONFIG(0x3FFF);

unsigned int TMR1HCOPY, TMR1LCOPY, TMR1X; // global variables - for use in all functions
unsigned char situatie = 0;

void main()
{
unsigned int TMROLD, FREQ; // local variables - only for use in main function

		TRISD = 0x00;			// leds aan PORTD
		TRISB = 0x01;			// RB0 = input
		ANSEL = 0x00;
		ANSELH = 0x00;			// all pins of PORTB digital (not analog)
        
		T1CON = 0b00110001;		//TMR1 - prescaler /8 - internal CLK - Start - timer loopt tegen 614.400Hz
		TMR1IE = 1;
		PEIE = 1; 

		OPTION_REG = 0b00000000;    // INT on BO on Falling edge (Bit 6) (voor oudere versies is dit OPTION) OPTION_REG = 0b01000000; 
		INTE = 1;           	// enable External int on B0
		
        GIE = 1;            	// enable global interrupts

        LCD_Start();			// start LCD initialisatie
		LCD_Clear();						// clear LCD
		LCD_PrintString("NEC DECODER");				// zet txt Hz op LCD
                              
    while(1) 
	{
	switch(situatie)
		{
		case 0:
		RD0=1;
		break;//jump to end
		
		case 1:
		RD1=1;
		break;//jump to end
		
		case 2:
		RD2=1;
		break;//jump to end
		
		case 3:
		RD3=1;
		break;//jump to end
		
		case 4:
		RD4=1;
		break;//jump to end

		case 5:
		RD5=1;
		break;//jump to end
		
		default: //else.
		RD6=1;
		break;
		}
	__delay_us(100);
	PORTD = 0x00;

     /*  									// flikkering LCD wegwerken
			LCD_Clear();						// clear LCD
			LCD_PrintNumber(FREQ);				// zet freq op LCD
			LCD_PrintString(" Hz");				// zet txt Hz op LCD

*/
    } 
}

/***** INTERRUPT SERVICE ROUTINE *****/

void interrupt isr(void)  
{
if (TMR1IF) // int op TMR1 overflow
	{
	TMR1IF = 0;
	situatie = 0;
	TMR1H = 0x00;			// clear TMR1L
	TMR1L = 0x00;			// clear TMR1H
	RD7 = ~RD7;
	}

else if (INTF)// interrupt in INT (Stijgende flank op RB0)
	{
	INTF = 0;				// Clear RB0 INT flag
	TMR1LCOPY = TMR1L;		// copieer TMR1L
	TMR1HCOPY = TMR1H; 		// copieer TMR1H
	TMR1H = 0x00;			// clear TMR1L
	TMR1L = 0x00;			// clear TMR1H
	TMR1X = ((TMR1HCOPY << 8)+(TMR1LCOPY)); // 16 bit Periode

	if ((TMR1X > 7500)&(TMR1X < 9000)) situatie = 1;
	else if ((TMR1X > 5000)&(TMR1X < 7499)) situatie = 2;
	else if ((TMR1X > 1050)&(TMR1X < 1700)) situatie = 3;
	else if ((TMR1X > 300)&(TMR1X < 1000)) situatie = 4;
	else situatie = 5;
	}
}